==> 0.1.0.tar.gz: permutizr.dart <==
library permutizr;
class Permutizr {
  int _length;
  var _list;
  int _total=1;
  int _parity=0;
  int _count=0;
  int _minorCount=0;
  int _base;
  Permutizr(this._length) {
    _list = new List(_length);
    if (_length>15)
      throw "length $_length too big, will take forever";
    if (_length<=0)
      throw "wrong length $_length";
    _base=_length-_generatedLength;
    for (int i=1;i<=_length;i++) {
      _total*=i;
      _list[i-1]=i;
    }  
  }
  List get list => _list;
  int get parity => _parity;
  bool nextSlow() {
    if (_count==0) { _count=1; return true;}
    if (_count==_total) return false;
    var nSwaps=0;
    
    // 1. finds the largest k, that _list[k] < _list[k+1]
    int first = _getFirst();
    if ( first == -1 ) {
      return false; // no greater permutation
    }
    
    // 2. find last index toSwap, that _list[k] < _list[toSwap]
    int toSwap = _list.length - 1;
    while ( _list[ first ] >= _list[ toSwap ] )
      --toSwap;
    // 3. _swap elements with indexes first and last
    _swap(first++, toSwap ); nSwaps++;
    // 4. reverse sequence from k+1 to n (inclusive) 
    toSwap = _list.length - 1;
    while ( first < toSwap ) {
      _swap(first++, toSwap-- );
      nSwaps++;
    }  
    _parity^=(nSwaps&1);
    return true;
  }

// finds the largest k, that _list[k] < _list[k+1]
// if no such k exists (there is not greater permutation), return -1
  int _getFirst() {
    for ( int i = _list.length - 2; i >= 0; --i )
      if ( _list[i] < _list[i+1] )
        return i;
    return -1;
  }

// swaps two elements (with indexes i and j) in array 
  void _swap(final int i, final int j ) {
    int tmp = _list[ i ];
    _list[ i ] = _list[ j ];
    _list[ j ] = tmp;
  }
  // generated code goes here
  // for some reason, switch statement with more cases is less efficient. Strange. Need to check how switch is implemented.
  static const int _generatedLength=3;
  bool next() {
    int save0, save1, save2;
    var c = _list;
    if (_count==0) { _count=1; return true;}
    if (_count==_total) return false;
    switch(_minorCount) {
    case 0:
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      _parity^=1;
      break;
    case 1:
      save0 = c[_base+0]; c[_base+0]=c[_base+2]; c[_base+2]=save0;
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      break;
    case 2:
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      _parity^=1;
      break;
    case 3:
      save0 = c[_base+0]; c[_base+0]=c[_base+1]; c[_base+1]=save0;
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      break;
    case 4:
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      _parity^=1;
      break;
    case 5:
      nextSlow();
      _minorCount =-1;
      break;
    }
    _minorCount++; _count++;
    return true;
  }
}

==> 0.1.1.tar.gz: permutizr.dart <==
library permutizr;
class Permutizr {
  int _length;
  var _list;
  int _total=1;
  //int _parity=0;
  int _count=0;
  int _minorCount=0;
  int _base;
  int _b0, _b1, _b2;
  Permutizr(this._length) {
    _list = new List(_length);
    if (_length>15)
      throw "length $_length too big, will take forever";
    if (_length<=0)
      throw "wrong length $_length";
    _base=_length-_generatedLength;
    _b0=_base; _b1=_base+1; _b2=_base+2;
    for (int i=1;i<=_length;i++) {
      _total*=i;
      _list[i-1]=i;
    }  
  }
  List get list => _list;
  //int get parity => _parity;
  bool nextSlow() {
    if (_count==0) { _count=1; return true;}
    if (_count==_total) return false;
    var nSwaps=0;
    
    // 1. finds the largest k, that _list[k] < _list[k+1]
    int first = _getFirst();
    if ( first == -1 ) {
      return false; // no greater permutation
    }
    
    // 2. find last index toSwap, that _list[k] < _list[toSwap]
    int toSwap = _list.length - 1;
    while ( _list[ first ] >= _list[ toSwap ] )
      --toSwap;
    // 3. _swap elements with indexes first and last
    _swap(first++, toSwap ); nSwaps++;
    // 4. reverse sequence from k+1 to n (inclusive) 
    toSwap = _list.length - 1;
    while ( first < toSwap ) {
      _swap(first++, toSwap-- );
      nSwaps++;
    }  
    //_parity^=(nSwaps&1);
    return true;
  }

// finds the largest k, that _list[k] < _list[k+1]
// if no such k exists (there is not greater permutation), return -1
  int _getFirst() {
    for ( int i = _list.length - 2; i >= 0; --i )
      if ( _list[i] < _list[i+1] )
        return i;
    return -1;
  }

// swaps two elements (with indexes i and j) in array 
  void _swap(final int i, final int j ) {
    int tmp = _list[ i ];
    _list[ i ] = _list[ j ];
    _list[ j ] = tmp;
  }
  // generated code goes here
  // for some reason, switch statement with more cases is less efficient. Strange. Need to check how switch is implemented.
  static const int _generatedLength=3;
  bool next() {
    int save0, save1, save2;
    var c = _list;
    switch(_minorCount) {
    case 0:
      if (_count==0) { _count=1; return true;}
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      //_parity^=1;
      break;
    case 1:
      save0 = c[_base+0]; c[_base+0]=c[_base+2]; c[_base+2]=save0;
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      break;
    case 2:
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      //_parity^=1;
      break;
    case 3:
      save0 = c[_base+0]; c[_base+0]=c[_base+1]; c[_base+1]=save0;
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      break;
    case 4:
      save1 = c[_base+1]; c[_base+1]=c[_base+2]; c[_base+2]=save1;
      //_parity^=1;
      break;
    case 5:
      if (_count==_total) {return false;};
      nextSlow();
      _minorCount =-1;
      break;
    }

    _minorCount++; _count++;
    return true;
  }
}
